@@ -250,12 +250,14 @@ int mutt_thread_set_flag (HEADER *cur, int flag, int bf, int subthread)
 int mutt_change_flag (HEADER *h, int bf)
 {
   int i, flag;
+  event_t event;
 
-  mvprintw (LINES - 1, 0, "? (D/N/O/r/*/!): ", bf ? _("Set %s flag") :
-	    _("Clear %s flag"));
+  mvprintw (LINES - 1, 0, "%s? (D/N/O/r/*/!): ", bf ? _("Set flag") : _("Clear flag"));
   clrtoeol ();
 
-  if ((i = mutt_getch ()) == ERR)
+  event = mutt_getch();
+  i = event.ch;
+  if (i == -1)
   {
     CLEARLINE (LINES-1);
     return (-1);
