@@ -1078,24 +1078,27 @@ void autoview_handler (BODY *a, STATE *s)
       mutt_message(_("Invoking autoview command: %s"),command);
     }
 
-    if (piped)
+    if ((fpin = safe_fopen (tempfile, "w+")) == NULL)
+    {
+      mutt_perror ("fopen");
+      rfc1524_free_entry (&entry);
+      return;
+    }
+    
+    mutt_copy_bytes (s->fpin, fpin, a->length);
+
+    if(!piped)
     {
-      thepid = mutt_create_filter (command, &fpin, &fpout, &fperr);
-      mutt_copy_bytes (s->fpin, fpin, a->length);
       fclose (fpin);
+      thepid = mutt_create_filter (command, NULL, &fpout, &fperr);
     }
     else
     {
-      if ((fpin = safe_fopen (tempfile, "w")) == NULL)
-      {
-        mutt_perror ("fopen");
-	rfc1524_free_entry (&entry);
-        return;
-      }
-
-      mutt_copy_bytes (s->fpin, fpin, a->length);
-      fclose (fpin);
-      thepid = mutt_create_filter (command, NULL, &fpout, &fperr);
+      unlink(tempfile);
+      fflush(fpin);
+      rewind(fpin);
+      thepid = mutt_create_filter_fd (command, NULL, &fpout, &fperr,
+				      fileno(fpin), -1, -1);
     }
 
     if (s->prefix)
@@ -1147,7 +1150,11 @@ void autoview_handler (BODY *a, STATE *s)
     fclose (fpout);
     fclose (fperr);
     mutt_wait_filter (thepid);
-    mutt_unlink (tempfile);
+    if(piped)
+      fclose(fpin);
+    else
+      mutt_unlink (tempfile);
+
     if (s->flags & M_DISPLAY) 
       mutt_clear_error ();
   }
