@@ -210,6 +210,7 @@ struct option_t MuttVars[] = {
 #ifdef USE_POP
   { "pop_delete",	DT_BOOL, R_NONE, OPTPOPDELETE, 0 },
   { "pop_host",		DT_STR,	 R_NONE, UL &PopHost, UL "" },
+  { "pop_last",		DT_BOOL, R_NONE, OPTPOPLAST, 0 },
   { "pop_port",		DT_NUM,	 R_NONE, UL &PopPort, 110 },
   { "pop_pass",		DT_STR,	 R_NONE, UL &PopPass, UL "" },
   { "pop_user",		DT_STR,	 R_NONE, UL &PopUser, UL "" },
@@ -332,6 +333,7 @@ struct command_t Commands[] = {
   { "color",		mutt_parse_color,	0 },
   { "uncolor",		mutt_parse_uncolor,	0 },
 #endif
+  { "exec",		mutt_parse_exec,	0 },
   { "fcc-hook",		mutt_parse_hook,	M_FCCHOOK },
   { "fcc-save-hook",	mutt_parse_hook,	M_FCCHOOK | M_SAVEHOOK },
   { "folder-hook",	mutt_parse_hook,	M_FOLDERHOOK },
