@@ -109,6 +109,12 @@ typedef struct
   int destroy;	/* destroy `data' when done? */
 } BUFFER;
 
+typedef struct
+{
+  int ch; /* raw key pressed */
+  int op; /* function op */
+} event_t;
+
 /* flags for _mutt_system() */
 #define M_DETACH_PROCESS	1	/* detach subprocess from group */
 
@@ -299,6 +305,7 @@ enum
   OPTPIPEDECODE,
   OPTPIPESPLIT,
   OPTPOPDELETE,
+  OPTPOPLAST,
   OPTPROMPTAFTER,
   OPTREADONLY,
   OPTRESOLVE,
